"""
    orgs
    ~~~~~~
    Request authenticated user's repos

    :copyright: (c) 2017 by Onni Software Ltd.
    :license: MIT License, see LICENSE for more details

"""

from gease.rest import Api

REPO_URL = "https://api.github.com/user/orgs"


class EndPoint(object):
    """
    Github authenticated user's repo endpoint
    """

    def __init__(self):
        self.__url = REPO_URL
        self.__client = Api.get_api()

    def get_all_organisations(self):
        json_reply = self.__client.get(self.__url)
        return json_reply



from nose.tools import eq_

from gease.uritemplate import UriTemplate, is_partial, extract_variables


def test_extract_variables():
    url = "{/abc}{/dde}"
    variables = extract_variables(url)
    eq_(variables, ["abc", "dde"])


def test_extract_variables_got_empty():
    url = ""
    variables = extract_variables(url)
    eq_(variables, [])


def test_uri_template_variable():
    template = UriTemplate("http://abc{/cute}")
    assert template.cute is None


def test_uri_template_template():
    template = UriTemplate("http://abc{/cute}")
    template.cute = "world"
    eq_(str(template), "http://abc/world")


def test_uri_template_template2():
    template = UriTemplate("http://abc{/cute}")
    eq_(template(cute="world"), "http://abc/world")


def test_uri_template_partial_apply():
    template = UriTemplate("http://abc{/cute}{/left}")
    eq_(template(cute="world"), "http://abc/world{/left}")


def test_is_partial():
    url = "{/abc}"
    assert is_partial(url) is True


def test_is_partial_2():
    url = UriTemplate("{/abc}")
    assert url.is_partial() is True


def test_get_original_template():
    original = "{/abc}"
    url = UriTemplate("{/abc}")
    assert url.get_template_string() is original



"""
    release
    ~~~~~~~~~~~~~~~~~~~

    Make github a release using github api

    :copyright: (c) 2017 by Onni Software Ltd.
    :license: MIT License, see LICENSE for more details

"""

import gease.exceptions as exceptions
from gease.orgs import EndPoint as Orgs
from gease.repo import EndPoint as Repo
from gease.rest import Api
from gease.uritemplate import UriTemplate

RELEASE_URL = "https://api.github.com/repos{/owner}{/repo}/releases"
KEY_HTML_URL = "html_url"
MESSAGE_MISSING_KEY = "No %s in github repsonse" % KEY_HTML_URL


class EndPoint(object):
    """
    Github release endpoint

    More documentation is available at
    https://developer.github.com/v3/repos/releases/
    """

    def __init__(self, owner, repo):
        self.__template = UriTemplate(RELEASE_URL)
        self.__template.owner = owner
        self.__template.repo = repo
        self.__client = Api.get_api()

    @property
    def url(self):
        return str(self.__template)

    def publish(self, **kwargs):
        """
        Publish the release

        More information at:
        https://developer.github.com/v3/repos/releases/#create-a-release

        :returns: the url to the release on github
        :throws: exception if gihub changes its api response
        """
        try:
            json_reply = self.__client.create(self.url, kwargs)
            return json_reply[KEY_HTML_URL]
        except exceptions.RepoNotFoundError:
            try:
                return self.republish(**kwargs)
            except exceptions.RepoNotFoundError:
                raise exceptions.AbnormalGithubResponse(
                    self.__template.repo + " does not exist!"
                )
        except KeyError:
            raise exceptions.AbnormalGithubResponse(MESSAGE_MISSING_KEY)
        except exceptions.ReleaseExistException:
            raise exceptions.AbnormalGithubResponse(
                "Release or tag %s exists" % kwargs["tag_name"]
            )
        except exceptions.UnhandledException as e:
            raise exceptions.AbnormalGithubResponse(str(e))

    def republish(self, **kwargs):
        repo = self.__template.repo
        org = which_org_has(repo)
        if org is None:
            raise exceptions.RepoNotFoundError()
        else:
            self.__template.owner = org
            json_reply = self.__client.create(self.url, kwargs)
            return json_reply[KEY_HTML_URL]


def which_org_has(repo):
    orgs = Orgs()
    for org_info in orgs.get_all_organisations():
        org_repo = Repo(org_info["repos_url"])
        for arepo in org_repo.get_all_repos():
            if repo == arepo["name"]:
                return org_info["login"]
    return None



import os

from mock import patch
from nose.tools import eq_, raises

import gease.constants as constants
import gease.exceptions as exceptions
from gease.utils import get_info


class TestMain:
    def setUp(self):
        self.patcher = patch("gease.utils.os.path.expanduser")
        self.fake_expand = self.patcher.start()
        self.fake_expand.return_value = os.path.join("tests", "fixtures")

    def tearDown(self):
        self.patcher.stop()

    def test_get_token(self):
        self.fake_expand.return_value = os.path.join("tests", "fixtures")
        user = get_info(constants.KEY_GEASE_TOKEN)
        eq_(user, "test")

    @raises(exceptions.NoGeaseConfigFound)
    def test_no_gease_file(self):
        self.fake_expand.return_value = os.path.join("tests")
        get_info(constants.KEY_GEASE_TOKEN)

    @raises(KeyError)
    def test_wrong_key(self):
        self.fake_expand.return_value = os.path.join(
            "tests", "fixtures", "malformed"
        )
        get_info(constants.KEY_GEASE_TOKEN)



"""
    uritemplate
    ~~~~~~~~~~~~~~~~~~~

    template github uri with variables

    :copyright: (c) 2017-2020 by Onni Software Ltd.
    :license: MIT License, see LICENSE for more details

"""

import re


class UriTemplate(object):
    """
    Yet another implemention of uri template.

    Examples:

       >>> from gease.uritemplate import UriTemplate
       >>> template = UriTemplate(
       ...     'https://github.com/repos{/user}{/repo}/releases')
       >>> template.variables
       ['user', 'repo']
       >>> template(user='chfw', repo='gease')
       'https://github.com/repos/chfw/gease/releases'

    Please note that if any of the two variables are not defined,
    the template becomes partial and can be instantiated as
    another UriTemplate.

       >>> template = UriTemplate(
       ...     'https://github.com/repos{/user}{/repo}/releases')
       >>> template(user='chfw')
       'https://github.com/repos/chfw{/repo}/releases'
       >>> template.is_partial()
       True
       >>> template(repo='gease')
       'https://github.com/repos/chfw/gease/releases'
       >>> template.is_partial()
       False

    """

    def __init__(self, url_template_string):
        self.__s = url_template_string
        self.__variables = extract_variables(self.__s)
        for v in self.__variables:
            self.__dict__[v] = None

    @property
    def variables(self):
        return self.__variables

    def get_template_string(self):
        return self.__s

    def is_partial(self):
        return is_partial(self.__str__())

    def __call__(self, **keywords):
        for key, value in keywords.items():
            if key in self.__variables:
                self.__dict__[key] = value
        return self.__str__()

    def __str__(self):
        templated = self.__s.format(**self.__get_dict())
        return templated

    def __get_dict(self):
        a_new_dict = {}
        for v in self.__variables:
            if self.__dict__[v]:
                a_new_dict["/" + v] = "/" + self.__dict__[v]
            else:
                a_new_dict["/" + v] = "{/" + v + "}"
        return a_new_dict


def extract_variables(url_template_string):
    results = re.findall(r"\{/([^}]+)\}", url_template_string)
    return results


def is_partial(url_template_string):
    return True if extract_variables(url_template_string) else False



from mock import MagicMock, patch
from nose.tools import eq_

from gease.orgs import EndPoint as Org
from gease.repo import EndPoint as Repo


class TestOrgEndPoint:
    def setUp(self):
        self.patcher = patch("gease.orgs.Api")
        self.fake_api_singleton = self.patcher.start()
        self.get = MagicMock()
        self.fake_api_singleton.get_api = MagicMock(
            return_value=MagicMock(get=self.get)
        )

    def tearDown(self):
        self.patcher.stop()

    def test_orgs(self):
        test_return = "hei"
        self.get.return_value = test_return

        org = Org()
        response = org.get_all_organisations()
        eq_(test_return, response)


class TestRepoEndPoint:
    def setUp(self):
        self.patcher = patch("gease.repo.Api")
        self.fake_api_singleton = self.patcher.start()
        self.get = MagicMock()
        self.fake_api_singleton.get_api = MagicMock(
            return_value=MagicMock(get=self.get)
        )

    def tearDown(self):
        self.patcher.stop()

    def test_repo(self):
        test_return = "hei"
        self.get.return_value = test_return

        repo = Repo()
        response = repo.get_all_repos()
        eq_(test_return, response)



from mock import MagicMock, patch
from nose.tools import raises

from gease.rest import Api
from gease.exceptions import (
    Forbidden,
    UrlNotFound,
    RepoNotFoundError,
    ReleaseExistException,
    AbnormalGithubResponse,
)

SAMPLE_422_ERROR = {
    "errors": [
        {"code": "already_exists", "field": "tag_name", "resource": "Release"}
    ],
    "documentation_url": "https://.../#create-a-release",
    "message": "Validation Failed",
}
WRONG_CREDENTIALS = {
    "message": "Bad credentials",
    "documentation_url": "https://developer.github.com/v3",
}


class TestApi:
    def setUp(self):
        self.patcher = patch("gease.rest.requests.Session")
        self.fake_session = self.patcher.start()

    def tearDown(self):
        self.patcher.stop()

    def test_create(self):
        self.fake_session.return_value = MagicMock(
            post=MagicMock(
                return_value=MagicMock(
                    status_code=201, json=MagicMock(return_value={})
                )
            )
        )
        api = Api("test")
        api.create("http://localhost/", "cool")

    @raises(ReleaseExistException)
    def test_existing_release(self):
        self.fake_session.return_value = MagicMock(
            post=MagicMock(
                return_value=MagicMock(
                    status_code=422,
                    json=MagicMock(return_value=SAMPLE_422_ERROR),
                )
            )
        )
        api = Api("test")
        api.create("http://localhost/", "cool")

    @raises(AbnormalGithubResponse)
    def test_wrong_credentials(self):
        self.fake_session.return_value = MagicMock(
            post=MagicMock(
                return_value=MagicMock(
                    status_code=401,
                    json=MagicMock(return_value=WRONG_CREDENTIALS),
                )
            )
        )
        api = Api("test")
        api.create("http://localhost/", "cool")

    @raises(RepoNotFoundError)
    def test_404(self):
        self.fake_session.return_value = MagicMock(
            post=MagicMock(
                return_value=MagicMock(
                    status_code=404,
                    json=MagicMock(return_value=WRONG_CREDENTIALS),
                )
            )
        )
        api = Api("test")
        api.create("http://localhost/", "cool")

    @raises(Exception)
    def test_unknown_error(self):
        self.fake_session.return_value = MagicMock(
            post=MagicMock(
                return_value=MagicMock(
                    status_code=400, json=MagicMock(return_value={})
                )
            )
        )
        api = Api("test")
        api.create("http://localhost/", "cool")

    @raises(UrlNotFound)
    def test_get_unknown_url(self):
        self.fake_session.return_value = MagicMock(
            get=MagicMock(side_effect=UrlNotFound)
        )
        api = Api("test")
        api.get("s")

    @raises(Forbidden)
    def test_get_forbidden_url(self):
        self.fake_session.return_value = MagicMock(
            get=MagicMock(side_effect=Forbidden)
        )
        api = Api("test")
        api.get("s")



from mock import MagicMock, patch
from nose.tools import eq_

from gease.contributors import EndPoint
from gease.exceptions import NoGeaseConfigFound


class TestPublish:
    @patch("gease.contributors.get_token")
    @patch("gease.contributors.Api.get_public_api")
    def test_all_contributors(self, fake_api, get_token):
        get_token.side_effect = [NoGeaseConfigFound]
        sample_reply = [
            {"login": "howdy", "url": "https://api.github.com/users/howdy"}
        ]
        fake_api.return_value = MagicMock(
            get=MagicMock(
                side_effect=[
                    sample_reply,
                    {"name": "hello world", "html_url": ""},
                ]
            )
        )

        repo = EndPoint("test", "repo")
        contributors = repo.get_all_contributors()

        eq_(
            contributors,
            [{"name": "hello world", "html_url": ""}],
        )

    @patch("gease.contributors.get_token")
    @patch("gease.contributors.Api.get_public_api")
    def test_private_api(self, fake_api, get_token):
        get_token.side_effect = [NoGeaseConfigFound]
        sample_reply = [
            {"login": "howdy", "url": "https://api.github.com/users/howdy"}
        ]
        fake_api.return_value = MagicMock(
            get=MagicMock(
                side_effect=[sample_reply, {"name": None, "html_url": ""}]
            )
        )

        repo = EndPoint("test", "repo")
        contributors = repo.get_all_contributors()

        eq_(
            contributors,
            [{"name": "howdy", "html_url": ""}],
        )

    @patch("gease.contributors.get_token")
    @patch("gease.contributors.Api.get_api")
    def test_no_names(self, fake_api, _):
        sample_reply = [
            {"login": "howdy", "url": "https://api.github.com/users/howdy"}
        ]
        fake_api.return_value = MagicMock(
            get=MagicMock(
                side_effect=[sample_reply, {"name": None, "html_url": ""}]
            )
        )

        repo = EndPoint("test", "repo")
        contributors = repo.get_all_contributors()

        eq_(
            contributors,
            [{"name": "howdy", "html_url": ""}],
        )



import os
import sys

import mock
from nose.tools import raises

import gease.exceptions as exceptions
from gease.main import main, fatal
from gease.constants import DEFAULT_RELEASE_MESSAGE

TEST_TAG = "tag"
SHORT_ARGS = ["gs", "repo", TEST_TAG]


class TestMain:
    def setUp(self):
        self.patcher = mock.patch("gease.utils.os.path.expanduser")
        self.fake_expand = self.patcher.start()
        self.fake_expand.return_value = os.path.join("tests", "fixtures")

        self.patcher2 = mock.patch("gease.main.EndPoint")
        self.fake_release = self.patcher2.start()

    def tearDown(self):
        self.patcher2.stop()
        self.patcher.stop()

    @raises(SystemExit)
    def test_key_error_in_main(self):
        self.fake_expand.return_value = os.path.join(
            "tests", "fixtures", "malformed"
        )
        with mock.patch.object(sys, "argv", SHORT_ARGS):
            main()

    @raises(SystemExit)
    def test_no_gease_file_in_main(self):
        self.fake_expand.return_value = os.path.join("tests")
        with mock.patch.object(sys, "argv", SHORT_ARGS):
            main()

    def test_good_commands(self):
        create_method = mock.MagicMock(
            return_value="http://localhost/tag/testurl"
        )
        self.fake_release.return_value = mock.MagicMock(publish=create_method)
        with mock.patch.object(sys, "argv", SHORT_ARGS):
            main()
            create_method.assert_called_with(
                tag_name=TEST_TAG, name=TEST_TAG, body=DEFAULT_RELEASE_MESSAGE
            )

    def test_custom_release_message(self):
        release_message = ["hello", "world", "you", "see", "it"]
        create_method = mock.MagicMock(
            return_value="http://localhost/tag/testurl"
        )
        self.fake_release.return_value = mock.MagicMock(publish=create_method)
        with mock.patch.object(sys, "argv", SHORT_ARGS + release_message):
            main()
            create_method.assert_called_with(
                tag_name=TEST_TAG,
                name=TEST_TAG,
                body=" ".join(release_message),
            )

    def test_quoted_release_message(self):
        release_message = "hello world you see it"
        create_method = mock.MagicMock(
            return_value="http://localhost/tag/testurl"
        )
        self.fake_release.return_value = mock.MagicMock(publish=create_method)
        with mock.patch.object(sys, "argv", SHORT_ARGS + [release_message]):
            main()
            create_method.assert_called_with(
                tag_name=TEST_TAG, name=TEST_TAG, body=release_message
            )

    @raises(SystemExit)
    def test_error_response(self):
        create_method = mock.MagicMock(
            side_effect=exceptions.AbnormalGithubResponse
        )
        self.fake_release.return_value = mock.MagicMock(publish=create_method)
        with mock.patch.object(sys, "argv", SHORT_ARGS):
            main()
            create_method.assert_called_with(
                tag_name=TEST_TAG, name=TEST_TAG, body=DEFAULT_RELEASE_MESSAGE
            )


@raises(SystemExit)
def test_no_args():
    with mock.patch.object(sys, "argv", []):
        main()


@raises(SystemExit)
def test_insufficent_args():
    with mock.patch.object(sys, "argv", SHORT_ARGS[:2]):
        main()


@raises(SystemExit)
def test_fatal_message():
    fatal("message and we quit")



DEFAULT_GEASE_FILE_NAME = ".gease"
DEFAULT_RELEASE_MESSAGE = "A new release via gease."
NOT_ENOUGH_ARGS = "Not enough arguments"
KEY_GEASE_USER = "user"
KEY_GEASE_TOKEN = "personal_access_token"
MESSAGE_FMT_RELEASED = "Release is created at: %s"



"""
    gease
    ~~~~~~~~~~~~~~~~~~~

    Make github release at command line

    :copyright: (c) 2017 by Onni Software Ltd.
    :license: MIT License, see LICENSE for more details

"""
import sys

import crayons

import gease.utils as utils
import gease.constants as constants
import gease.exceptions as exceptions
from gease.release import EndPoint
from gease._version import __version__, __description__

HELP = """%s. version %s

Usage: %s

where:

   release message is optional. It could be a quoted string or space separate
   string

Examples:

   gs gease v0.0.1 first great release
   gs gease v0.0.2 "second great release"
""" % (
    crayons.yellow("gease " + __description__),
    crayons.magenta(__version__, bold=True),
    crayons.yellow("gs repo tag [release message]", bold=True),
)


def main():
    if len(sys.argv) < 3:
        if len(sys.argv) == 2:
            error(constants.NOT_ENOUGH_ARGS)
        print(HELP)
        sys.exit(-1)

    repo = sys.argv[1]
    tag = sys.argv[2]
    msg = " ".join(sys.argv[3:])
    try:
        user = get_default_user()
        if len(msg) == 0:
            msg = constants.DEFAULT_RELEASE_MESSAGE
        release = EndPoint(user, repo)
        url = release.publish(tag_name=tag, name=tag, body=msg)
        print(constants.MESSAGE_FMT_RELEASED % crayons.green(url))
    except exceptions.RepoNotFoundError:
        fatal("%s does not exist!" % repo)
    except exceptions.AbnormalGithubResponse as e:
        fatal(str(e))
    except exceptions.NoGeaseConfigFound as e:
        fatal(str(e))
    except exceptions.ReleaseExistException:
        fatal("Release %s exists" % tag)
    except KeyError as e:
        fatal("Key %s is not found" % str(e))


def get_default_user():
    return utils.get_info(constants.KEY_GEASE_USER)


def error(message):
    print("Error: %s" % crayons.red(message))


def fatal(message):
    error(message)
    sys.exit(-1)


if __name__ == "__main__":
    main()



"""
    contributors
    ~~~~~~~~~~~~~
    get a list of contributors

    :copyright: (c) 2020 by Onni Software Ltd.
    :license: MIT License, see LICENSE for more details

"""
import sys

import crayons

import gease.utils as utils
import gease.constants as constants
import gease.exceptions as exceptions
from gease.rest import Api, get_token
from gease._version import __version__
from gease.uritemplate import UriTemplate

HELP = """%s. version %s

Usage: %s

Where:
   user/org is the your github username or orgnisation name
   repo is the repository name

Examples:

    contributors pyexcel pyexcel-io
""" % (
    crayons.yellow("contributors list the contributors of a repo"),
    crayons.magenta(__version__, bold=True),
    crayons.yellow("contributors user/org repo", bold=True),
)
REPO_URL = "https://api.github.com/repos{/owner}{/repo}/contributors"


class EndPoint(object):
    """
    Github authenticated user's repo endpoint
    """

    def __init__(self, owner, repo):
        self.__template = UriTemplate(REPO_URL)
        self.__template.owner = owner
        self.__template.repo = repo
        try:
            get_token()
            self.__client = Api.get_api()
        except exceptions.NoGeaseConfigFound:
            self.__client = Api.get_public_api()

    @property
    def url(self):
        return str(self.__template)

    def get_all_contributors(self):
        json_reply = self.__client.get(self.url)
        contributors = []
        for user in json_reply:
            user_details = self.__client.get(user["url"])
            user_name = user_details["name"]
            if user_name is None:
                user_details["name"] = user["login"]
            contributors.append(user_details)
        return contributors


def main():
    if len(sys.argv) < 3:
        if len(sys.argv) == 2:
            error(constants.NOT_ENOUGH_ARGS)
        print(HELP)
        sys.exit(-1)

    user = sys.argv[1]
    repo = sys.argv[2]
    try:
        repo = EndPoint(user, repo)
        message = repo.get_all_contributors()
        print(message)
    except exceptions.RepoNotFoundError:
        fatal("%s does not exist!" % repo)
    except exceptions.AbnormalGithubResponse as e:
        fatal(str(e))
    except exceptions.NoGeaseConfigFound as e:
        fatal(str(e))
    except KeyError as e:
        fatal("Key %s is not found" % str(e))


def get_default_user():
    return utils.get_info(constants.KEY_GEASE_USER)


def error(message):
    print("Error: %s" % crayons.red(message))


def fatal(message):
    error(message)
    sys.exit(-1)


if __name__ == "__main__":
    main()



from mock import MagicMock, patch
from nose.tools import eq_, raises

import gease.exceptions as exceptions
from gease.release import EndPoint


class TestPublish:
    def setUp(self):
        self.patcher = patch("gease.release.Api")
        self.fake_api_singleton = self.patcher.start()
        self.fake_api = MagicMock()
        self.fake_api_singleton.get_api = self.fake_api
        self.patcher2 = patch("gease.rest.get_token")
        self.fake_token = self.patcher2.start()
        self.fake_token.return_value = "token"

    def tearDown(self):
        self.patcher2.stop()
        self.patcher.stop()

    def test_create_release(self):
        self.fake_api.return_value = MagicMock(
            create=MagicMock(return_value={"html_url": "aurl"})
        )
        release = EndPoint("owner", "repo")
        release.publish(hello="world")

    @raises(exceptions.AbnormalGithubResponse)
    def test_unknown_error(self):
        self.fake_api.return_value = MagicMock(
            create=MagicMock(return_value={})
        )
        release = EndPoint("owner", "repo")
        release.publish(hello="world")

    @raises(exceptions.AbnormalGithubResponse)
    def test_release_exist(self):
        self.fake_api.return_value = MagicMock(
            create=MagicMock(side_effect=exceptions.ReleaseExistException)
        )
        release = EndPoint("owner", "repo")
        release.publish(hello="world", tag_name="existing tag")

    @raises(exceptions.AbnormalGithubResponse)
    def test_repo_not_found(self):
        self.fake_api.return_value = MagicMock(
            create=MagicMock(side_effect=exceptions.RepoNotFoundError)
        )
        release = EndPoint("owner", "repo")
        release.republish = MagicMock(side_effect=exceptions.RepoNotFoundError)
        release.publish(hello="world")

    @raises(exceptions.AbnormalGithubResponse)
    def test_unhandled_exception(self):
        self.fake_api.return_value = MagicMock(
            create=MagicMock(side_effect=exceptions.UnhandledException)
        )
        release = EndPoint("owner", "repo")
        release.publish(hello="world")


class TestRepublish:
    def setUp(self):
        self.patcher = patch("gease.release.Orgs")
        self.fake_orgs = self.patcher.start()
        self.patcher2 = patch("gease.release.Repo")
        self.fake_repo = self.patcher2.start()
        self.patcher3 = patch("gease.release.Api")
        self.fake_api_singleton = self.patcher3.start()
        self.fake_api = MagicMock()
        self.fake_api_singleton.get_api = self.fake_api

    def tearDown(self):
        self.patcher3.stop()
        self.patcher2.stop()
        self.patcher.stop()

    def test_create_release(self):
        test_url = "special url"
        self.fake_orgs.return_value = MagicMock(
            get_all_organisations=MagicMock(
                return_value=[{"repos_url": "repo", "login": "zhangfei"}]
            )
        )
        self.fake_repo.return_value = MagicMock(
            get_all_repos=MagicMock(return_value=[{"name": "repo"}])
        )
        self.fake_api.return_value = MagicMock(
            create=MagicMock(return_value={"html_url": test_url})
        )
        release = EndPoint("owner", "repo")
        ret = release.republish(hello="world")
        eq_(ret, test_url)



import os
import json

import gease.constants as constants
import gease.exceptions as exceptions

try:
    FileNotFoundError
except NameError:
    FileNotFoundError = IOError


def get_info(key):
    """
    Find geasefile from user's home folder
    """
    home_dir = os.path.expanduser("~")
    geasefile = os.path.join(home_dir, constants.DEFAULT_GEASE_FILE_NAME)
    try:
        with open(geasefile, "r") as config:
            gease = json.load(config)
            return gease[key]
    except FileNotFoundError:
        raise exceptions.NoGeaseConfigFound("Cannot find %s" % geasefile)



"""
    rest
    ~~~~~~~~~~~~~~~~~~~

    Only use post interface

    :copyright: (c) 2017-2020 by Onni Software Ltd.
    :license: MIT License, see LICENSE for more details

"""

import requests

import gease.utils as utils
import gease.constants as constants
import gease.exceptions as exceptions


class Api(object):
    """
    A session holder so that each request shares the same token
    """

    __instance = None

    def __init__(self, personal_access_token):
        self.__session = requests.Session()
        if personal_access_token:
            self.__session.headers.update(
                {"Authorization": "token %s" % personal_access_token}
            )

    def create(self, url, data):
        """
        Do a post to the url
        """
        r = self.__session.post(url, json=data)
        if r.status_code == 201:
            return r.json()
        elif r.status_code == 422:
            raise exceptions.ReleaseExistException()
        elif r.status_code == 401:
            response = r.json()
            message = "%s. Please check your gease file" % response["message"]
            raise exceptions.AbnormalGithubResponse(message)
        elif r.status_code == 404:
            raise exceptions.RepoNotFoundError()
        else:
            message = "Github responded with HTTP %s, %s " % (
                r.status_code,
                r.text,
            )
            raise exceptions.UnhandledException(message)

    def get(self, url):
        r = self.__session.get(url)
        if r.status_code == 200:
            return r.json()
        elif r.status_code == 404:
            raise exceptions.UrlNotFound(f"{url} does not exist")
        elif r.status_code == 403:
            raise exceptions.Forbidden(f"{url} does not exist")
        else:
            raise exceptions.UnhandledException(f"{r.status_code} is received")

    @classmethod
    def get_api(cls):
        if cls.__instance is None:
            token = get_token()
            cls.__instance = cls(token)
        return cls.__instance

    @classmethod
    def get_public_api(cls):
        cls.__instance = cls(None)
        return cls.__instance


def get_token():
    """
    Find geasefile from user's home folder
    """
    return utils.get_info(constants.KEY_GEASE_TOKEN)
